/*
m_cubicon - A 3d morphing wavetable oscillator

DESCRIPTION
An oscillator holding a cube of waveforms through which the user can browse in
x, y and z direction. Optionally waveforms can be morphed or switched.

SYNTAX
aout m_cubicon kpitch, kmx, kmy, kmz, kmorph, isnaptab, ipostab, iconftab, \
	iquality

INITIALIZATION
isnaptab - Number of an ftable holding all the waveforms for a cube
	(see supplied material, include with something like:
	f1 0 3456 -23 "angel_cube1.ftable"
ipostab - Table mapping positions of waveforms in the cube (use supplied
	cube_pos.ftable:
	f1 0 27 -23 "cube_pos.ftable"
iconftab - Number of an ftable containing numbers of ftables defining
	interpolation curves (see supplied .part files), choose 0 for complete
	linear fade (volume mixing of the waves)
iquality - Synthesis quality
	0 = use adsynt (no interpolation)
	non-zero = use adsynt2 (linear audio interpolation)

PERFORMANCE
kpitch - Pitch of the oscillator in Hz.
kmx, kmy, kmz - x, y and z coordinates in the cube (0-3, internally wrapped round)
kmorph - If 0 switch waveforms, if non-zero morph between waveforms

CREDITS
Author: Jeanette C., all supplied wavetables are self generated both
algorithmically and from recordings.
*/

opcode m_cubicon, a, kkkkkiiii
	kpitch, kmx, kmy, kmz, kmorph, isnaptab, ipostab, iconftab, iquality xin


	aout init 0
	if kmorph == 0 then ; step the control signals using only whole numbers
		kmx = int(kmx)
		kmy = int(kmy)
		kmz = int(kmz)
	endif

	; Wrap the signals to fit the range 0 to 3
	kmx wrap kmx, 0, 3
	kmy wrap kmy, 0, 3
	kmz wrap kmz, 0, 3

	; Finally divide the signals to fit hvs' 0-1 range
	kmx = kmx / 3
	kmy = kmy / 3
	kmz = kmz / 3
	
	; Initialise some HVS variables and necessary ftables
	inumparams init 128 ; Number of harmonics (elements/params for HVS)
	inumpointsx init 4 ; Number of points along the x-axis
	inumpointsy init 4 ; Number of points along the y-axis
	inumpointsz init 4 ; number of points along tye z-axis
	iouttab ftgentmp 0, 0, 128, -2, 0 ; Output table for harmonic amplitudes
	ifreqs ftgenonce 0, 0, 128, -7, 1, 128, 129 ; Table holding harmonic freqs
	isine ftgenonce 0, 0, 32768, 10, 1 ; A sine wave for additive synthesis

	; Morph the parameters/spectra in the cube
	hvs3 kmx, kmy, kmz, inumparams, inumpointsx, inumpointsy, inumpointsz, iouttab, ipostab, isnaptab, iconftab

	if iquality == 0 then ; use adsynt for synthesis
		aout adsynt 0.5, kpitch, isine, ifreqs, iouttab, 128
	else ; use adsynt2 for synthesis
		aout adsynt 0.5, kpitch, isine, ifreqs, iouttab, 128
	endif
	xout aout
endop
